'use strict';

exports.__esModule = true;
exports.PRELOAD_METADATA_METHOD_NAME = exports.GET_METADATA_METHOD_NAME = undefined;

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = withReactRouterMetadata;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _reactDisplayName = require('react-display-name');

var _reactDisplayName2 = _interopRequireDefault(_reactDisplayName);

var _reactRouter = require('react-router');

var _reactHtmlMetadata = require('react-html-metadata');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var GET_METADATA_METHOD_NAME = exports.GET_METADATA_METHOD_NAME = 'getMetadata';
var PRELOAD_METADATA_METHOD_NAME = exports.PRELOAD_METADATA_METHOD_NAME = 'preloadMetadata';

function withReactRouterMetadata() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var _Object$assign = (0, _assign2.default)({
        mapParamsToProps: function mapParamsToProps(params) {
            return params;
        },
        staticMethodName: PRELOAD_METADATA_METHOD_NAME,
        componentStaticMethodName: GET_METADATA_METHOD_NAME,
        metadataPropName: _reactHtmlMetadata.METADATA_PROP_NAME
    }, options),
        mapParamsToProps = _Object$assign.mapParamsToProps,
        staticMethodName = _Object$assign.staticMethodName,
        componentStaticMethodName = _Object$assign.componentStaticMethodName,
        metadataPropName = _Object$assign.metadataPropName;

    return function (Component) {
        var componentName = (0, _reactDisplayName2.default)(Component);
        var getMetadata = Component[componentStaticMethodName];
        (0, _invariant2.default)(typeof getMetadata === 'function', 'Component ' + componentName + ' requires a static function named \'' + componentStaticMethodName + '\' to use withReactRouterMetadata().');

        var ReactRouterMetadata = function (_Component) {
            (0, _inherits3.default)(ReactRouterMetadata, _Component);

            function ReactRouterMetadata(props, context) {
                (0, _classCallCheck3.default)(this, ReactRouterMetadata);

                var _this = (0, _possibleConstructorReturn3.default)(this, _Component.call(this, props, context));

                _this.state = {
                    currentMetadata: {}
                };
                return _this;
            }

            ReactRouterMetadata.prototype.setMetadata = function setMetadata() {
                var nextMetadata = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

                this.props.metadata.update(this.state.currentMetadata, nextMetadata || {});
                if (nextMetadata !== null) {
                    this.setState({ currentMetadata: nextMetadata });
                }
            };

            ReactRouterMetadata.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
                // eslint-disable-next-line no-unused-vars
                var location = nextProps.location,
                    match = nextProps.match,
                    history = nextProps.history,
                    props = (0, _objectWithoutProperties3.default)(nextProps, ['location', 'match', 'history']);

                this.setMetadata(getMetadata({ location: location, match: match }, props));
            };

            ReactRouterMetadata.prototype.componentWillMount = function componentWillMount() {
                // eslint-disable-next-line no-unused-vars
                var _props = this.props,
                    location = _props.location,
                    match = _props.match,
                    history = _props.history,
                    props = (0, _objectWithoutProperties3.default)(_props, ['location', 'match', 'history']);

                this.setMetadata(getMetadata({ location: location, match: match }, props));
            };

            ReactRouterMetadata.prototype.componentWillUnmount = function componentWillUnmount() {
                this.setMetadata();
            };

            ReactRouterMetadata.prototype.render = function render() {
                var _props2 = this.props,
                    match = _props2.match,
                    location = _props2.location,
                    history = _props2.history,
                    metadata = _props2.metadata,
                    componentProps = (0, _objectWithoutProperties3.default)(_props2, ['match', 'location', 'history', 'metadata']);

                return _react2.default.createElement(Component, componentProps);
            };

            return ReactRouterMetadata;
        }(Component);

        // Static action method, using name as defined in options


        ReactRouterMetadata[staticMethodName] = function (routeProps, actionParams, routerCtx) {
            var htmlMetadata = actionParams[metadataPropName],
                params = (0, _objectWithoutProperties3.default)(actionParams, [metadataPropName]);

            if (typeof htmlMetadata === 'undefined') {
                // No htmlMetadata instance, no metadata will be pre-loaded.
                // - this can be used to prevent pre-loading on client renders
                return;
            }

            // Verify valid metadata type
            (0, _invariant2.default)(htmlMetadata instanceof _reactHtmlMetadata.Metadata, 'actionParams requires prop ' + metadataPropName + ' to be an instance of Metadata.');

            var props = mapParamsToProps(params, routerCtx);
            htmlMetadata.appendMetadata(getMetadata(routeProps, props));
        };

        ReactRouterMetadata.displayName = 'withReactRouterMetadata(' + componentName + ')';

        var hoisted = (0, _hoistNonReactStatics2.default)(ReactRouterMetadata, Component);
        return (0, _reactRouter.withRouter)((0, _reactHtmlMetadata.withMetadata)('metadata')(hoisted));
    };
}